<?php declare(strict_types=1);

namespace Lewisdale\Webmentions\Router;

use Exception;

/**
 * This is a dumb, custom Router I slapped together as a way of reminding myself how to PHP.
 * 
 * It's not great, and currently on supports two methods: GET and POST.
 * 
 * Usage is as follows:
 * 
 * ```
 * $router = new Lewisdale\Webmentions\Router\Router();
 * $router->get('/a/get/route', function($request, $response) {
 *  $response->status_code = StatusCode::OK;
 *  return "Hello world!";
 * });
 * 
 * $router->get('/a/json/route', function($request, $response) {
 *  return $response->json(new MyClass());
 * });
 * 
 * $router->dispatch();
 * ```
 * 
 * TL;DR? Don't use this. There are some good routing libraries already.
 * 
 * But if you're reading this, it's already too late.
 */
class Router {
    private array $routes;

    private function trim_route(string $route): string {
        $route = $route == "/" ? $route : rtrim($route, "/");
        $route = str_replace("/", "\/", $route);

        return $route;
    }

    private function format_route(string $route): string {
        $formatted = $this->trim_route($route);
        return "#^$formatted$#";      
    }

    public function get(string $route, callable $fn) {
        $this->routes[] = new Route(Method::GET, $this->format_route($route), $fn);
    }

    public function post(string $route, callable $fn) {
        $this->routes[] = new Route(Method::POST, $this->format_route($route), $fn);
    }

    private function get_params(array $matches): array {
        return array_map(
            function($match) { return $match[0]; },
            array_shift($matches)
        );
    }

    public function dispatch() {
        $uri = $_SERVER['REQUEST_URI'];
        $method = Method::from($_SERVER['REQUEST_METHOD']);

        $num_matched = 0;

        $response = new Response();

        foreach($this->routes as $route) {
            if ($method == $route->type) {
                $matches = array();
                if (preg_match_all($route->pattern, $uri, $matches)) {
                    $num_matched++;
                    $fn = $route->fn;
                    $params = $this->get_params($matches);
                    $response->status_code = StatusCode::Ok; // Default status code to OK for the "average" route
                    try {
                        $response->body .= $fn($this->construct_request_object($params), $response);
                    } catch (Exception) {
                        $response->status_code = StatusCode::InternalError;
                        // TODO: Log the error
                    }
                }
            }
        }

        if (!$num_matched) {
            // Handle 404
            $response->status_code = StatusCode::NotFound;
        }

        $this->respond($response);
    }


    private function respond(Response $response) {
        http_response_code($response->status_code->code());

        echo $response->body;
    }
    
    private function construct_request_object(array $params)
    {
        $uri = $_SERVER['REQUEST_URI'];
        $method = Method::from($_SERVER['REQUEST_METHOD']);
        $query = $_GET;
        $post = $_POST;

        return new Request($params, $uri, $method, $post, $query);
    }
}

?>